#include "callback.h"
#include "debug.h"

Callback::Callback():
    func(0),
    numArgs(0),
    rvalue(false)
{
    args[0]=args[1]=args[2]=0;
}

Callback::Callback(funcType0 f):
    func(reinterpret_cast<void*>(f)),
    numArgs(0),
    rvalue(true)
{
    args[0]=args[1]=args[2]=0;
}

Callback::Callback(void* f, void* a0):
    func(f),
    numArgs(1),
    rvalue(true)
{
    args[0]=a0;
    args[1]=args[2]=0;
}

Callback::Callback(void* f, void* a0, void* a1):
    func(f),
    numArgs(2),
    rvalue(true)
{
    args[0]=a0;
    args[1]=a1;
    args[2]=0;
}

Callback::Callback(void* f, void* a0, void* a1, void* a2):
    func(f),
    numArgs(3),
    rvalue(true)
{
    args[0]=a0;
    args[1]=a1;
    args[2]=a2;
}

Callback::~Callback()
{
    if(!rvalue && args[0])
        free(args[0]);
}

void Callback::operator()()
{
    switch(numArgs)
    {
    case 0:
        (*reinterpret_cast<funcType0>(func))();
        break;
        
    case 1:
        (*reinterpret_cast<funcType1>(func))(args[0]);
        break;
        
    case 2:
        (*reinterpret_cast<funcType2>(func))(args[0], args[1]);
        break;
        
    case 3:
        (*reinterpret_cast<funcType3>(func))(args[0], args[1], args[2]);
        break;
    }
}

void Callback::clear()
{
    ZCASSERT(!rvalue);
    
    if(args[0])
        free(args[0]);
    
    args[0]=args[1]=args[2]=0;
    numArgs=0;
    func=0;
}

Callback& Callback::operator=(const Callback& other)
{
    if(args[0])
        free(args[0]);
    
    func=other.func;
    numArgs=other.numArgs;
    args[0]=other.args[0];
    args[1]=other.args[1];
    args[2]=other.args[2];
    return *this;
}

Callback& Callback::operator=(const funcType0 newFunc)
{
    if(args[0])
        free(args[0]);
    
    func=reinterpret_cast<void*>(newFunc);
    numArgs=0;
    args[0]=0;
    args[1]=0;
    args[2]=0;
    return *this;
}
